/*
 * Copyright (c) 2014 Jonas 'Sortie' Termansen.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * unistd/execlpe.c
 * Loads a program image.
 */

#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>

int execlpe(const char* filename, const char* argv0, ...)
{
	va_list args;
	va_start(args, argv0);
	size_t numargs = 0;
	if ( argv0 )
	{
		numargs++;
		while ( va_arg(args, const char*) )
			numargs++;
	}
	(void) va_arg(args, char* const*) /* envp */;
	va_end(args);

	char** argv = (char**) malloc(sizeof(char*) * (numargs + 1));
	if ( !argv )
		return -1;

	va_start(args, argv0);
	argv[0] = (char*) argv0;
	for ( size_t i = 1; i <= numargs; i++ )
		argv[i] = (char*) va_arg(args, const char*);
	char* const* envp = va_arg(args, char* const*);
	va_end(args);

	int result = execvpe(filename, argv, envp);

	free(argv);

	return result;
}
